@@ -1,7 +1,8 @@
 
+from __future__ import division
 import os
 
-from PyQt4.QtGui import QIcon, QPixmap, QImage
+from PyQt4.QtGui import QIcon, QPixmap, QImage, QApplication
 from PyQt4.QtCore import QDir, QString
 
 from ConfigParser import ConfigParser, SafeConfigParser, NoSectionError
@@ -16,6 +17,7 @@ class DataParser:
         self.tree_controller = ui_handler.tree_controller
         self.controller = ui_handler.controller
         self.logger = logger
+        self.uid = None
 
     def parse_metadata(self, data, opened_folders):
         tree_item = None
@@ -47,6 +49,8 @@ class DataParser:
                     child = Collection(path, modified, icon, has_thumb, parent_root)
                 else:
                     child = Resource(path, size, modified, item["mime_type"], icon, has_thumb, parent_root)
+                    if path.startswith("/Public/"):
+                        child.generate_public_link(self.uid)
                 folder.add_item(child)
                 self.check_deleted(item, child)
 
@@ -101,6 +105,7 @@ class DataParser:
     def parse_account_info(self, resp):
         if resp.status == 200:
             account_data = resp.data
+            self.uid = str(account_data["uid"])
             name = account_data["display_name"]
             self.ui_handler.manager_ui.label_username.setText(name)
             user_icon = QPixmap(self.controller.datahandler.datapath("ui/icons/user.png"))
@@ -121,15 +126,20 @@ class MaemoDataHandler:
         self.controller = controller
         self.maemo = maemo
         self.logger = logger
+        self.store_auth_to_file = True
+        self.dont_show_dl_dialog = False
+        self.only_sync_on_wlan = True
         if self.maemo:
-            home = QDir.home().absolutePath()
+            self.user_home = str(QDir.home().absolutePath())
             self.app_root = "/opt/dropn900/"
-            self.config_root = str(home) + "/.dropn900/"
-            self.data_root = str(home) + "/MyDocs/DropN900/"
+            self.config_root = self.user_home + "/.dropn900/"
+            self.data_root = self.user_home + "/MyDocs/DropN900/"
+            self.default_data_root = self.user_home + "/MyDocs/DropN900"
         else:
             self.app_root = ""
             self.config_root = ""
             self.data_root = ""
+            self.default_data_root = ""
 
     def datapath(self, datafile):
         return self.app_root + datafile
@@ -137,11 +147,14 @@ class MaemoDataHandler:
     def configpath(self, configfile):
         return self.config_root + configfile
         
+    def datadirpath(self, datafile):
+        return self.data_root + datafile
+    
     def get_data_dir_path(self):
         return self.data_root
         
     def startup_checks(self):
-        self.logger.info("Maemo startup checks...")
+        self.logger.info("Running Maemo startup checks...")
         if QDir.home().absolutePath() != "/home/user":
             self.logger.warning(">> home folder != /home/user")
         self.check_for_data_folder()
@@ -150,17 +163,26 @@ class MaemoDataHandler:
     def check_for_data_folder(self):
         if not self.maemo:
             return
-        data_dir = QDir.home()
-        if data_dir.cd("MyDocs"):
-            if not data_dir.cd("DropN900"):
-                if data_dir.mkdir("DropN900"):
-                    self.logger.info(">> Created default data dir " + str(data_dir.absolutePath() + "/DropN900"))
+        if self.data_root == (self.user_home + "/MyDocs/DropN900/"):
+            data_dir = QDir.home()
+            if data_dir.cd("MyDocs"):
+                if not data_dir.cd("DropN900"):
+                    if data_dir.mkdir("DropN900"):
+                        self.logger.info(">> Created default data dir " + str(data_dir.absolutePath() + "/DropN900"))
+                    else:
+                        self.logger.error(">> Could not create default data dir 'DropN900' to " + str(data_dir.absolutePath()))
                 else:
-                    self.logger.error(">> Could not create default data dir 'DropN900' to " + str(data_dir.absolutePath()))
+                    self.logger.info(">> Default data dir: " + str(data_dir.absolutePath()))
             else:
-                self.logger.info(">> Default data dir " + str(data_dir.absolutePath()) + " OK")
+                self.logger.error(">> ERROR - Could not find 'MyDocs' folder from " + str(data_dir.absolutePath()))
         else:
-            self.logger.error(">> ERROR - Could not find 'MyDocs' folder from " + str(data_dir.absolutePath()))
+            non_default_data_dir = QDir(self.data_root)
+            if non_default_data_dir.exists():
+                self.logger.info(">> Default data dir: " + self.data_root)
+            else:
+                self.logger.warning(">> User set default data dir " + self.data_root + " does not exist, resetting to default")
+                self.data_root = self.user_home + "/MyDocs/DropN900/"
+                self.check_for_data_folder()
 
     def check_for_config_folder(self):
         if not self.maemo:
@@ -171,31 +193,54 @@ class MaemoDataHandler:
                 self.logger.info(">> Created config dir '.dropn900' to " + str(config_dir.absolutePath()))
             else:
                 self.logger.error(">> Could not create config dir '.dropn900' to " + str(config_dir.absolutePath()))
-        else:
-            self.logger.info(">> Config dir " + str(config_dir.absolutePath()) + " OK")
 
     def store_auth(self, access_token):
-        token_config = ConfigParser()
-        token_config.add_section("token")
-        token_config.set("token", "secret", access_token.secret)
-        token_config.set("token", "key", access_token.key)
-        try:
-            config_file = open(self.configpath("token.ini"), "w")
-            token_config.write(config_file)
-            config_file.close()
-            self.logger.auth("Stored received access token")
-        except IOError:
-            self.logger.error("I/O error while storing received access token, file " + self.configpath("token.ini"))
+        if self.store_auth_to_file:
+            token_config = ConfigParser()
+            token_config.add_section("token")
+            token_config.set("token", "secret", access_token.secret)
+            token_config.set("token", "key", access_token.key)
+            try:
+                config_file = open(self.configpath("token.ini"), "w")
+                token_config.write(config_file)
+                config_file.close()
+                self.logger.auth("Stored received access token")
+            except IOError:
+                self.logger.config("I/O error while storing received access token, file " + self.configpath("token.ini"))
+        else:
+            self.logger.auth("Skipping access token storing due to user settings")
         
     def reset_auth(self):
         try:
             os.remove(self.configpath("token.ini"))
             self.logger.auth("Authentication reseted")
+            self.controller.ui.show_banner("Authentication reseted")
         except OSError:
-            pass
-        self.controller.start_trusted_auth()
+            self.controller.ui.show_banner("No stored access token, could not reset authentication")
         
+    def copy_url_to_clipboard(self, url):
+        clipboard = QApplication.clipboard()
+        clipboard.setText(url)
+        self.controller.ui.show_banner("Copied " + url + " to clipboard", 3000)
         
+    """ Code snipped taken from http://code.activestate.com/recipes/577081-humanized-representation-of-a-number-of-bytes/ """
+    def humanize_bytes(self, bytes, precision = 1):
+        abbrevs = (
+            (1<<50L, 'PB'),
+            (1<<40L, 'TB'),
+            (1<<30L, 'GB'),
+            (1<<20L, 'MB'),
+            (1<<10L, 'KB'),
+            (1, 'bytes')
+        )
+        if bytes == 1:
+            return '1 byte'
+        for factor, suffix in abbrevs:
+            if bytes >= factor:
+                break
+        return '%.*f %s' % (precision, bytes / factor, suffix)
+
+
 """ Parent class for all data items """
 
 class Item:
@@ -203,7 +248,7 @@ class Item:
     def __init__(self, path, root, modified, icon, has_thumb):
         self.path = path
         self.root = root
-        self.modified = modified.split(" +")[0]
+        self.format_modified(modified)
         self.icon = icon
         self.has_thumb = has_thumb
         
@@ -225,6 +270,48 @@ class Item:
         else:
             return self.size
 
+    """ This may seem a bit difficult of an approach, let me explain:
+        We get english names for months from dropbox, this will get us into localization problems
+        if the device has != english as language. We form same kind of timestamps that we get from maemo os module.
+        This way we can compare timestamps directly when syncing """
+    def format_modified(self, modified):
+        if modified == "":
+            self.modified = modified
+            return
+
+        months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
+        # Strinp timezone (always +0000) and name of day
+        modified = modified.split(" +")[0]
+        modified = modified[5:]
+        
+        # Split timestamp to elements
+        modified_split = modified.split(" ")
+        if len(modified_split) == 4:
+            # Get elements
+            day = modified_split[0]
+            month_string = modified_split[1]
+            year =  modified_split[2]
+            time = modified_split[3]
+        else:
+            print "#1 FATAL ERROR ON TIMESTAMP PARSING!"
+            self.modified = "<timestamp parse error, report to app author>"
+            return
+            
+        # Convert name of english month name to number
+        try:
+            month = months.index(month_string) + 1
+            if month < 10:
+                month = "0" + str(month)
+            else:
+                month = str(month)
+        except IndexError:
+            print "#2 FATAL ERROR ON TIMESTAMP PARSING!"
+            self.modified = "<timestamp parse error, report to app author>"
+            return
+            
+        # Form final timestamp
+        self.modified = day + "." + month + "." + year + " " + time
+        
     def get_modified(self):
         return self.modified
     
@@ -233,6 +320,8 @@ class Item:
         self.load_animation = animation
 
     def set_loading(self, loading):
+        if self.load_widget == None or self.load_animation == None:
+            return
         try:
             self.load_widget.setVisible(loading)
             if loading:
@@ -347,6 +436,13 @@ class Resource(Item):
 
     def is_folder(self):
         return False
+        
+    def generate_public_link(self, uid):
+        if uid != None:
+            public_path = self.path[len("/Public"):]
+            self.public_link = "http://dl.dropbox.com/u/" + uid + public_path
+        else:
+            self.public_link = None
     
     # For debug prints
     def __str__(self):
