@@ -14,6 +14,8 @@ from uicontroller import UiController
 from data import MaemoDataHandler
 from connectionmanager import ConnectionManager
 from logger import Logger
+from settings import ConfigHelper, SettingsWidget
+from transfers import SyncManager, TransferManager, TransferWidget
 
 """ Main programs controller, instantiates all the nessesary classes, starts ui and auth """
 
@@ -21,16 +23,33 @@ class DropN900(QtCore.QObject):
 
     def __init__(self, debug_mode = False, maemo_env = True):
         self.logger = Logger(debug_mode)
-        # Setup data handler
+        
+        # Setup data handler and config helper
         self.datahandler = MaemoDataHandler(self, maemo_env, self.logger)
-        # Start ui
+        self.config_helper = ConfigHelper(self.datahandler, self.logger)
+        
+        # Start and setup ui
         self.ui = UiController(self, debug_mode, self.logger)
         self.logger.set_ui(self.ui)
-        # Setup connection
+        self.settings_widget = SettingsWidget(self.ui, self.config_helper, self.logger)
+
+        # Setup connection and pass to elements that need it
         self.connection = ConnectionManager(self, self.ui, self.logger, maemo_env)
-        # Pass tree controller the connection
-        self.ui.tree_controller.setup(self.connection)
+        #self.ui.tree_controller.setup(self.connection)
+        #self.settings_widget.setup(self.connection)
         
+        # Create transfer managers and widget
+        self.sync_manager = SyncManager(self)
+        self.transfer_manager = TransferManager(self)
+        self.transfer_widget = TransferWidget(self.transfer_manager)
+
+        # Pass objects around
+        self.ui.tree_controller.setup(self.connection)
+        self.settings_widget.setup(self.connection, self.sync_manager)
+        self.ui.set_settings_widget(self.settings_widget)
+        self.ui.set_transfer_widget(self.transfer_widget)
+        self.transfer_manager.set_transfer_widget(self.transfer_widget)
+            
     def start(self):
         # Show ui
         self.ui.show()
@@ -50,14 +69,14 @@ class DropN900(QtCore.QObject):
             access_key = token_config.get("token", "key")
             access_secret = token_config.get("token", "secret")
             if access_key != "" and access_secret != "":
-                self.logger.auth("Found existing access token")
+                self.logger.config("Found existing access token")
                 self.login_done = True
                 self.init_dropbox_client(oauth.OAuthToken(access_key, access_secret))
             else:
                 self.logger.error("Parsing access token from file failed")
                 self.datahandler.reset_auth()
         except NoSectionError:
-            self.logger.auth("No stored access token found")
+            self.logger.config("No stored access token found")
             self.start_trusted_auth()
     
     def start_trusted_auth(self):
@@ -65,6 +84,7 @@ class DropN900(QtCore.QObject):
         self.ui.switch_context("trustedlogin")
     
     def end_trusted_auth(self, email, password):
+        self.login_done = False
         if self.connection.connection_available():
             if self.authenticator:
                 del self.authenticator
@@ -72,15 +92,17 @@ class DropN900(QtCore.QObject):
             try:
                 access_token = self.authenticator.obtain_trusted_access_token(email, password)
             except AssertionError:
-                self.ui.trusted_login_ui.label_error.setText("Email and/or password invalid")
+                self.ui.set_trusted_login_error("Email and/or password invalid")
                 return
             except socket.gaierror:
+                self.ui.set_trusted_login_info("Requesting a network connection...")
                 self.connection.request_connection()
                 return
             self.login_done = True
             self.datahandler.store_auth(access_token)
             self.init_dropbox_client(access_token)
         else:
+            self.ui.set_trusted_login_info("Requesting a network connection...")
             self.connection.request_connection()
     
     def init_dropbox_client(self, access_token):
@@ -97,7 +119,7 @@ class DropN900(QtCore.QObject):
     def get_config(self):
         return auth.Authenticator.load_config(self.datahandler.datapath(".config"))
 
-            
+
 """ This is the main function that starts the program
 
     * Set debug if you want to get stdout prints from python
